@@ -13,7 +13,7 @@ import iotas.config_manager
 from iotas.database import Database
 from iotas.migration_assistant import MigrationAssistant
 from iotas.note_database import NoteDatabase
-from iotas.preferences_window import PreferencesWindow
+from iotas.preferences_dialog import PreferencesDialog
 from iotas.nextcloud_sync_worker import NextcloudSyncWorker
 from iotas.sync_manager import SyncManager
 from iotas.widgets import load_widgets
@@ -57,6 +57,7 @@ class Application(Adw.Application):
         self.cursors = {}
         self.__actions = {}
         self.__backup_manager = None
+        self.__preferences_dialog = None
         self.__previous_version = ""
 
         self.connect("startup", self.__on_startup)
@@ -139,7 +140,7 @@ class Application(Adw.Application):
         if self.development_mode or self.__debug_session:
             loglevel = logging.DEBUG
 
-            for module in ("urllib3", "gtkspellcheck", "markdown_it"):
+            for module in ("urllib3", "gtkspellcheck", "markdown_it", "pypandoc"):
                 logging.getLogger(module).setLevel(logging.INFO)
 
         logging.basicConfig(
@@ -151,11 +152,13 @@ class Application(Adw.Application):
         if "create-backup" in options:
             self.register()
             if self.get_property("is-remote"):
-                # Translators: Log message
-                msg = _("Please quit the running instance of Iotas before creating the backup")
-                logging.error(msg)
-                self.quit()
-                success = False
+                self.activate_action("create-backup")
+                logging.info("Requesting backup from already running Iotas")
+                logging.info(
+                    "Note: backup outcome will be logged from the other process; the return code "
+                    "here doesn't reflect the backup result"
+                )
+                success = True
             else:
                 success = self.__backup_manager.create_backup()
             self.quit()
@@ -163,9 +166,9 @@ class Application(Adw.Application):
         elif "restore-backup" in options:
             self.register()
             if self.get_property("is-remote"):
-                # Translators: Log message
-                msg = _("Please quit the running instance of Iotas before restoring the backup")
-                logging.error(msg)
+                logging.error(
+                    "Please quit the running instance of Iotas before restoring the backup"
+                )
                 self.quit()
                 success = False
             else:
@@ -177,8 +180,7 @@ class Application(Adw.Application):
             if self.get_property("is-remote"):
                 self.activate_action("quit-running")
             else:
-                # Translators: Log message
-                logging.warning(_("No running instance found"))
+                logging.warning("No running instance found")
             self.quit()
             return 0
         elif "new-note" in options:
@@ -207,12 +209,10 @@ class Application(Adw.Application):
         elif "toggle-extended-preferences" in options:
             self.register()
             if iotas.config_manager.get_show_extended_preferences():
-                # Translators: Log message
-                logging.info(_("Hiding extended preferences"))
+                logging.info("Hiding extended preferences")
                 iotas.config_manager.set_show_extended_preferences(False)
             else:
-                # Translators: Log message
-                logging.info(_("Showing extended preferences"))
+                logging.info("Showing extended preferences")
                 iotas.config_manager.set_show_extended_preferences(True)
             self.quit()
             return 0
@@ -337,7 +337,7 @@ class Application(Adw.Application):
 
         add_action("about", self.__show_about_dialog)
         add_action("quit", self.__on_quit_shortcut, "<Control>q")
-        add_action("settings", self.__show_preferences_window, "<Control>comma")
+        add_action("settings", self.__show_preferences_dialog, "<Control>comma")
         add_action("shortcuts", self.__show_shortcuts_window, "<Control>question")
         add_action("create-note-wrapper", self.__on_create_note_wrapper)
         add_action("open-note-wrapper", self.__on_open_note_wrapper, None, GLib.VariantType("u"))
@@ -348,6 +348,7 @@ class Application(Adw.Application):
             parameter_type=GLib.VariantType("s"),
         )
         add_action("quit-running", self.__on_quit_running)
+        add_action("create-backup", self.__on_create_backup)
 
     def __create_window(self) -> None:
         self.__window = Window(self, self.db, self.__sync_manager)
@@ -384,15 +385,21 @@ class Application(Adw.Application):
         builder = Gtk.Builder()
         builder.add_from_resource("/org/gnome/World/Iotas/about_dialog.ui")
         about_dialog = builder.get_object("AboutDialog")
-        about_dialog.set_transient_for(self.__window)
-        about_dialog.present()
+        about_dialog.present(self.__window)
 
     def __show_shortcuts_window(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        win = KeyboardShortcutsWindow(self.__window)
-        win.present()
+        KeyboardShortcutsWindow(self.__window).present()
 
-    def __show_preferences_window(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        PreferencesWindow().present()
+    def __show_preferences_dialog(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        if self.__preferences_dialog is not None:
+            return
+
+        self.__preferences_dialog = PreferencesDialog()
+        self.__preferences_dialog.connect("closed", self.__on_preferences_dialog_closed)
+        self.__preferences_dialog.present(self.__window)
+
+    def __on_preferences_dialog_closed(self, _dialog: GObject.Object) -> None:
+        self.__preferences_dialog = None
 
     def __set_style_variant(self, _action: Gio.SimpleAction, new_style: str) -> None:
         iotas.config_manager.set_style(new_style.get_string())
@@ -424,6 +431,10 @@ class Application(Adw.Application):
     def __on_search_wrapper(self, _action: Gio.SimpleAction, param: GLib.Variant) -> None:
         self.__window.activate_action("win.search-from-cli", param)
 
+    def __on_create_backup(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        logging.info("Running remotely requested backup")
+        self.__backup_manager.create_backup()
+
     def __apply_index_category_label_css(self) -> None:
         # First pass approach to building this is using a display CSS provider, instead of adding
         # providers to each label. Presumably this is better, but it's a mess.
