@@ -11,6 +11,8 @@ from typing import Optional
 from iotas.category_header_bar import CategoryHeaderBar
 from iotas.category_treeview_list_store import CategoryTreeViewListStore
 import iotas.config_manager
+from iotas.export_dialog import ExportDialog
+from iotas.focus_mode_helper import FocusModeHelper
 from iotas.font_size_selector import FontSizeSelector
 from iotas.note import Note
 from iotas.theme_selector import ThemeSelector
@@ -32,7 +34,8 @@ class Editor(Adw.Bin):
     FONT_SETTING_KEYS = ["monospace-font-name", "document-font-name"]
     HEADERBAR_HIDE_DELAY = 2000
 
-    _title_button = Gtk.Template.Child()
+    _title_label = Gtk.Template.Child()
+    _is_dirty = Gtk.Template.Child()
     _headerbar_stack = Gtk.Template.Child()
     _main_header_bar = Gtk.Template.Child()
     _rename_header_bar = Gtk.Template.Child()
@@ -56,15 +59,17 @@ class Editor(Adw.Bin):
     def __init__(self):
         super().__init__()
 
+        self.__active = False
         self.__note = None
         self.__updating_from_remote = False
         self.__sync_authenticated = False
         self.__buffer_handlers = []
         self.__current_note_handlers = []
         self.__render_view = None
-        self.__render_view_added = False
-        self.__active = False
         self.__font_families = {}
+        self.__html_generator = None
+        self.__focus_mode_action = None
+        self.__cancel_action = None
 
         self.__auto_hide_headerbar = False
         self.__auto_hide_headerbar_fullscreen_only = False
@@ -80,10 +85,6 @@ class Editor(Adw.Bin):
 
         self._sourceview.connect("size-allocated", update_top_margin_box_height)
 
-        # Retain whether markdown is enabled for the session. At the time of writing instantiating
-        # WebKit after startup was crashing, so we wait until restart to change the behaviour.
-        self.__markdown_render_enabled = iotas.config_manager.get_markdown_render_enabled()
-
         self._category_header_bar.connect("categories-changed", self.__category_changed)
         self._category_header_bar.connect("abort", self.__abort_category_change)
 
@@ -95,15 +96,35 @@ class Editor(Adw.Bin):
         self._rename_header_bar.connect("renamed", self.__on_title_renamed)
         self._rename_header_bar.connect("cancelled", self.__on_rename_cancelled)
 
-        self._render_edit_button_stack.set_visible(self.__markdown_render_enabled)
-        if self.__markdown_render_enabled:
-            self.__init_webkit()
+        self._render_edit_button_stack.set_visible(
+            iotas.config_manager.get_markdown_render_enabled()
+        )
 
         self.__categories_model = None
 
         style_manager = Adw.StyleManager.get_default()
         style_manager.connect("notify::dark", self.__on_dark_style_updated)
 
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.MARKDOWN_SYNTAX_HIGHLIGHTING}",
+            self.__on_syntax_highlighting_changed,
+        )
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.EDITOR_THEME}",
+            self.__on_theme_changed,
+        )
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.MARKDOWN_RENDER}",
+            self.__on_render_enabled_changed,
+        )
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.MARKDOWN_KEEP_WEBKIT_PROCESS}",
+            self.__on_keep_webkit_changed,
+        )
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.MARKDOWN_TEX_SUPPORT}",
+            self.__on_tex_support_changed,
+        )
         iotas.config_manager.settings.connect(
             f"changed::{iotas.config_manager.LINE_LENGTH}", self.__on_line_length_changed
         )
@@ -147,6 +168,8 @@ class Editor(Adw.Bin):
             iotas.config_manager.get_hide_editor_headerbar_when_fullscreen()
         )
 
+        self.__focus_mode_helper = FocusModeHelper(self._sourceview.get_buffer())
+
     def setup(self, model: CategoryTreeViewListStore) -> None:
         """Perform initial setup.
 
@@ -186,16 +209,10 @@ class Editor(Adw.Bin):
         handler_id = buffer.connect_after("insert-text", self.__on_insert_text)
         self.__buffer_handlers.append(handler_id)
 
-        if iotas.config_manager.get_markdown_syntax_hightlighting_enabled():
-            language = GtkSource.LanguageManager.get_default().get_language("iotas-markdown")
-            buffer.set_language(language)
-            buffer.set_highlight_syntax(True)
-        else:
-            buffer.set_language(None)
-            buffer.set_highlight_syntax(False)
-
+        self.__setup_buffer_highlighting()
         self.__setup_note_signals()
         self.__update_title()
+        self.__update_dirty()
 
         self._read_only_banner.set_revealed(note.read_only)
         self._sourceview.set_editable(not note.read_only)
@@ -225,10 +242,14 @@ class Editor(Adw.Bin):
 
         self.__check_and_hide_headerbar_after_delay()
 
+        if self.__focus_mode_action.get_state():
+            self.__focus_mode_helper.active = True
+
         # Handle initialising in view mode
         if (
             iotas.config_manager.get_markdown_render_enabled()
             and iotas.config_manager.get_markdown_default_to_render()
+            and note.content.strip() != ""
         ):
             # Prevent a flash of the editor while loading rendered view
             self._sourceview.set_visible(False)
@@ -254,6 +275,8 @@ class Editor(Adw.Bin):
             self.__note.disconnect(handler)
         self.__current_note_handlers = []
 
+        self.__focus_mode_helper.active = False
+
         self.current_note = None
 
         # If cleaning up WebKit process after each render do that now on closing the note.
@@ -297,12 +320,13 @@ class Editor(Adw.Bin):
         if self._render_edit_stack.get_visible_child() == self._editor_scrolledwin:
             self._sourceview.grab_focus()
 
-            # Temporary ugly workaround for Gtk hidden cursor issue, hackhackhack
-            # https://gitlab.gnome.org/GNOME/gtk/-/issues/6405
-            if not self._sourceview.get_buffer().get_has_selection():
-                self._sourceview.do_move_cursor(
-                    self._sourceview, Gtk.MovementStep.LOGICAL_POSITIONS, 0, False
-                )
+    def update_for_dialog_visibility(self, visible: bool) -> None:
+        """Update if dialogs visible, disabling actions.
+
+        :param bool visible: Whether a dialog is visible
+        """
+        if self.active:
+            self.__cancel_action.set_enabled(not visible)
 
     @GObject.Property(type=bool, default=False)
     def active(self) -> bool:
@@ -419,6 +443,7 @@ class Editor(Adw.Bin):
         action = Gio.SimpleAction.new("go-back-or-cancel")
         action.connect("activate", self.__on_cancel)
         action_group.add_action(action)
+        self.__cancel_action = action
         app.set_accels_for_action("editor.go-back-or-cancel", ["Escape", "<Alt>Left"])
 
         action = Gio.SimpleAction.new("clear-category")
@@ -431,6 +456,11 @@ class Editor(Adw.Bin):
         action_group.add_action(action)
         app.set_accels_for_action("editor.create-new-note", ["<Control>n"])
 
+        action = Gio.SimpleAction.new("export-note")
+        action.connect("activate", self.__on_export_note)
+        action_group.add_action(action)
+        app.set_accels_for_action("editor.export-note", ["<Control><Shift>s"])
+
         action = Gio.SimpleAction.new("delete-note")
         action.connect("activate", self.__on_delete_note)
         action_group.add_action(action)
@@ -445,15 +475,10 @@ class Editor(Adw.Bin):
         action_group.add_action(action)
         app.set_accels_for_action("editor.decrease-line-length", ["<Control>Down"])
 
-        action = Gio.SimpleAction.new("increase-line-length")
-        action.connect("activate", self.__on_increase_line_length)
+        action = Gio.SimpleAction.new("toggle-line-length-limit")
+        action.connect("activate", self.__on_line_length_limit_toggle)
         action_group.add_action(action)
-        app.set_accels_for_action("editor.increase-line-length", ["<Control>Up"])
-
-        action = Gio.SimpleAction.new("decrease-line-length")
-        action.connect("activate", self.__on_decrease_line_length)
-        action_group.add_action(action)
-        app.set_accels_for_action("editor.decrease-line-length", ["<Control>Down"])
+        app.set_accels_for_action("editor.toggle-line-length-limit", ["<Shift><Control>l"])
 
         action = Gio.SimpleAction.new("increase-font-size")
         action.connect("activate", self.__on_increase_font_size)
@@ -472,6 +497,17 @@ class Editor(Adw.Bin):
         action_group.add_action(action)
         app.set_accels_for_action("editor.reset-font-size", ["<Control>0"])
 
+        action = Gio.SimpleAction.new_stateful("focus-mode", None, GLib.Variant("b", False))
+        action.connect("change-state", self.__on_focus_mode_toggle)
+        action_group.add_action(action)
+        app.set_accels_for_action("editor.focus-mode", ["<Control><Shift>f"])
+        self.__focus_mode_action = action
+
+        action = Gio.SimpleAction.new("show-menu")
+        action.connect("activate", self.__on_show_menu)
+        action_group.add_action(action)
+        app.set_accels_for_action("editor.show-menu", ["F10"])
+
         app.get_active_window().insert_action_group("font-size-selector", action_group)
         self.__action_group = action_group
 
@@ -490,6 +526,16 @@ class Editor(Adw.Bin):
         if enabled and self.__note is not None and self.__note.read_only:
             self.__update_readonly_actions()
 
+    def __setup_buffer_highlighting(self) -> None:
+        buffer = self._sourceview.get_buffer()
+        if iotas.config_manager.get_markdown_syntax_hightlighting_enabled():
+            language = GtkSource.LanguageManager.get_default().get_language("iotas-markdown")
+            buffer.set_language(language)
+            buffer.set_highlight_syntax(True)
+        else:
+            buffer.set_language(None)
+            buffer.set_highlight_syntax(False)
+
     def __on_insert_text(
         self, _buffer: GtkSource.Buffer, location: Gtk.TextIter, char: str, length: int
     ) -> None:
@@ -528,32 +574,24 @@ class Editor(Adw.Bin):
         delete_end.forward_char()
         buffer.delete(match_end, delete_end)
 
-    def __on_web_view_loaded(self, _obj: GObject.Object) -> None:
-        # Attempt to reduce time spent with blank view visible
-        if not self.__render_view_added:
-            self.__render_view.show()
-            self._render_edit_stack.add_child(self.__render_view)
-            self.__render_view_added = True
-
-        gesture = Gtk.GestureSingle()
-        gesture.connect("end", self.__on_render_single_click)
-        self.__render_view.add_controller(gesture)
-
-        controller = Gtk.EventControllerKey()
-        controller.connect("key-pressed", self.__on_render_key_pressed)
-        self.__render_view.add_controller(controller)
-
-        self._render_edit_button_stack.set_visible_child(self._edit_button)
-        self._render_edit_stack.set_visible_child(self.__render_view)
+    def __on_webview_loaded(self, _obj: GObject.Object) -> None:
+        if self.__render_view.exporting:
+            self.__render_view.exporting = False
+        else:
+            self._render_edit_button_stack.set_visible_child(self._edit_button)
+            if not self.__render_view.get_visible():
+                self.__render_view.set_visible(True)
+            self._render_edit_stack.set_visible_child(self.__render_view)
 
     def __on_enter_search(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        self.__enter_search(False, False)
+        if not self._search_header_bar.active:
+            self.__enter_search(resuming=False, for_replace=False)
 
     def __on_search_resumed(self, _object: GObject.Object) -> None:
-        self.__enter_search(True, False)
+        self.__enter_search(resuming=True, for_replace=False)
 
     def __on_enter_replace(self, _object: GObject.Object) -> None:
-        self.__enter_search(False, True)
+        self.__enter_search(resuming=False, for_replace=True)
 
     def __on_title_renamed(self, _obj: GObject.Object, new_title: str) -> None:
         if self.__note.title != new_title:
@@ -584,25 +622,93 @@ class Editor(Adw.Bin):
         if self._headerbar_stack.get_visible_child() == self._category_header_bar:
             self._category_header_bar.clear_and_apply()
 
+    def __on_export_note(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        if self.__html_generator is None:
+            from iotas.html_generator import HtmlGenerator
+
+            self.__html_generator = HtmlGenerator()
+            self.__push_font_updates()
+
+        dialog = ExportDialog(self.__note, self.__html_generator, self.__ensure_webkit_initialised)
+        dialog.present(self)
+
     def __on_delete_note(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
         self.emit("note-deleted", self.__note)
 
+    def __on_syntax_highlighting_changed(self, _settings: Gio.Settings, _key: str) -> None:
+        if self.active:
+            self.__setup_buffer_highlighting()
+
+    def __on_theme_changed(self, _settings: Gio.Settings, _key: str) -> None:
+        if self.active and iotas.config_manager.get_markdown_syntax_hightlighting_enabled():
+            self.__update_scheme_and_dark_style()
+
+    def __on_render_enabled_changed(self, _settings: Gio.Settings, _key: str) -> None:
+        enabled = iotas.config_manager.get_markdown_render_enabled()
+        self._render_edit_button_stack.set_visible(enabled)
+        if (
+            not enabled
+            and self.active
+            and self._render_edit_stack.get_visible_child() == self.__render_view
+        ):
+            self.__toggle_render(force=True)
+
+    def __on_keep_webkit_changed(self, _settings: Gio.Settings, _key: str) -> None:
+        keep = iotas.config_manager.get_markdown_keep_webkit_process()
+        if not keep and (
+            not self.active
+            or self._render_edit_stack.get_visible_child() == self._editor_scrolledwin
+        ):
+            if self.__render_view is not None:
+                logging.debug("Calling terminate for WebKit process")
+                self.__render_view.terminate_web_process()
+
+    def __on_tex_support_changed(self, _settings: Gio.Settings, _key: str) -> None:
+        if self.active and self._render_edit_stack.get_visible_child() == self.__render_view:
+            self.__render_view.render_retaining_scroll(self.__note, None)
+
     def __on_line_length_changed(self, _settings: Gio.Settings, _key: str) -> None:
         length = iotas.config_manager.get_line_length()
         logging.debug(f"Line length now {length}px")
         self.__refresh_line_length_from_setting()
 
     def __on_increase_line_length(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        length = iotas.config_manager.get_line_length() + self.LINE_LENGTH_STEP
-        if length < self.get_width():
-            iotas.config_manager.set_line_length(length)
-            self.__notify_line_length_change(length)
+        setting_max = iotas.config_manager.get_line_length_max()
+        current_length = iotas.config_manager.get_line_length()
+        if current_length == setting_max:
+            # Translators: Description, notification
+            self.__notify_setting_change(_("Line length limit already disabled"))
+            return
+        else:
+            new_length = current_length + self.LINE_LENGTH_STEP
+        if new_length < self.get_width() - 2 * self._sourceview.BASE_MARGIN:
+            iotas.config_manager.set_line_length(new_length)
+            self.__notify_line_length_change(new_length)
+        else:
+            iotas.config_manager.set_line_length(setting_max)
+            # Translators: Description, notification
+            self.__notify_setting_change(_("Line length limit disabled"))
 
     def __on_decrease_line_length(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        length = iotas.config_manager.get_line_length() - self.LINE_LENGTH_STEP
-        if length >= self.LINE_LENGTH_STEP:
-            iotas.config_manager.set_line_length(length)
-            self.__notify_line_length_change(length)
+        setting_max = iotas.config_manager.get_line_length_max()
+        current_length = iotas.config_manager.get_line_length()
+        new_length = current_length - self.LINE_LENGTH_STEP
+        if current_length == setting_max:
+            new_length = min(self.get_width(), iotas.config_manager.get_default_line_length())
+            iotas.config_manager.set_line_length(new_length)
+            self.__notify_line_length_change(new_length)
+        elif new_length >= self.LINE_LENGTH_STEP:
+            iotas.config_manager.set_line_length(new_length)
+            self.__notify_line_length_change(new_length)
+
+    def __on_line_length_limit_toggle(
+        self, _action: Gio.SimpleAction, _param: GLib.Variant
+    ) -> None:
+        setting_maximum = iotas.config_manager.get_line_length_max()
+        if iotas.config_manager.get_line_length() == setting_maximum:
+            iotas.config_manager.set_line_length(iotas.config_manager.get_default_line_length())
+        else:
+            iotas.config_manager.set_line_length(setting_maximum)
 
     def __on_font_family_changed(self, _settings: Gio.Settings, _key: str) -> None:
         self.__load_font_family_from_setting()
@@ -677,6 +783,13 @@ class Editor(Adw.Bin):
     ) -> None:
         self.__update_title()
 
+    def __on_note_dirty_changed(
+        self,
+        _obj: GObject.Object,
+        _value: GObject.ParamSpec,
+    ) -> None:
+        self.__update_dirty()
+
     def __on_dark_style_updated(
         self,
         _obj: GObject.Object,
@@ -708,6 +821,10 @@ class Editor(Adw.Bin):
         self.__font_size_selector.reset()
         self.__notify_font_size()
 
+    def __on_focus_mode_toggle(self, action: Gio.SimpleAction, state: bool) -> None:
+        action.set_state(state)
+        self.__focus_mode_helper.active = state
+
     def __on_create_new_note(
         self,
         _obj: GObject.Object,
@@ -724,15 +841,34 @@ class Editor(Adw.Bin):
 
         self.activate_action("index.create-note-with-content", param)
 
+    def __on_show_menu(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        self._menu_button.popup()
+
     def __init_webkit(self) -> None:
         # Lazy import module in an attempt to reduce startup performance hit on devices with
         # render view disabled
         from iotas.markdown_render_view import MarkdownRenderView
 
+        if self.__html_generator is None:
+            from iotas.html_generator import HtmlGenerator
+
+            self.__html_generator = HtmlGenerator()
+
         self.__render_view = MarkdownRenderView()
         self.__render_view.connect("checkbox-toggled", self.__on_checkbox_toggled)
-        self.__render_view.connect("loaded", self.__on_web_view_loaded)
-        self.__render_view.setup()
+        self.__render_view.connect("loaded", self.__on_webview_loaded)
+        self.__render_view.setup(self.__html_generator)
+        self._render_edit_stack.add_child(self.__render_view)
+
+        gesture = Gtk.GestureSingle()
+        gesture.connect("end", self.__on_render_single_click)
+        self.__render_view.add_controller(gesture)
+
+        controller = Gtk.EventControllerKey()
+        controller.connect("key-pressed", self.__on_render_key_pressed)
+        self.__render_view.add_controller(controller)
+
+        self.__push_font_updates()
 
         self._render_search_header_bar.bind_property(
             "active", self.__render_view, "searching", GObject.BindingFlags.SYNC_CREATE
@@ -748,7 +884,7 @@ class Editor(Adw.Bin):
     def __setup_note_signals(self) -> None:
         self.__current_note_handlers = []
 
-        handler_id = self.__note.connect("notify::dirty", self.__on_note_update_from_sync)
+        handler_id = self.__note.connect("notify::dirty", self.__on_note_dirty_changed)
         self.__current_note_handlers.append(handler_id)
         handler_id = self.__note.connect("notify::title", self.__on_note_update_from_sync)
         self.__current_note_handlers.append(handler_id)
@@ -773,27 +909,30 @@ class Editor(Adw.Bin):
         GLib.idle_add(self.emit, "note-modified", self.__note)
 
     def __update_title(self) -> None:
-        title = self.__note.title
-        if self.__sync_authenticated and self.__note.dirty and title != "":
-            title = f"•  {title}"
-        self._title_button.set_label(title)
-        self._title_button.get_child().props.ellipsize = Pango.EllipsizeMode.END
+        self._title_label.set_label(self.__note.title)
 
-    def __update_content_from_remote(self, note: Note) -> None:
+    def __update_dirty(self) -> None:
+        show_dirty = self.__sync_authenticated and self.__note.dirty and self.__note.title != ""
+        self._is_dirty.set_visible(show_dirty)
+
+    def __update_content_from_remote(self, _note: Note) -> None:
         if self.__note and not self.__note.dirty:
             self.set_sensitive(False)
             self.__updating_from_remote = True
             editor_scroll_value = self._editor_scrolledwin.get_vadjustment().get_value()
-            self._sourceview.get_buffer().set_text(self.__note.content)
+            buffer = self._sourceview.get_buffer()
+            insert_iter = buffer.get_iter_at_mark(buffer.get_insert())
+            pre_offset = insert_iter.get_offset()
+            buffer.set_text(self.__note.content)
 
-            def delayed_editor_scroll_reset():
+            def delayed_editor_scroll_and_cursor_reset():
                 self._editor_scrolledwin.get_vadjustment().set_value(editor_scroll_value)
+                if len(self.__note.content) > pre_offset:
+                    buffer.place_cursor(buffer.get_iter_at_offset(pre_offset))
 
-            GLib.idle_add(delayed_editor_scroll_reset)
+            GLib.idle_add(delayed_editor_scroll_and_cursor_reset)
             if self._render_edit_stack.get_visible_child() == self.__render_view:
-                self.__render_view.render_retaining_scroll(
-                    self.__note.content, self.__note.read_only
-                )
+                self.__render_view.render_retaining_scroll(self.__note, None)
             self.__updating_from_remote = False
             self.set_sensitive(True)
 
@@ -865,23 +1004,23 @@ class Editor(Adw.Bin):
         self.__check_and_hide_headerbar_after_delay()
         self._render_search_header_bar.exit()
 
-    def __toggle_render(self) -> None:
+    def __toggle_render(self, force: bool = False) -> None:
         """Toggle whether showing rendered view."""
 
-        if not self.__markdown_render_enabled:
+        if not iotas.config_manager.get_markdown_render_enabled() and not force:
             return
 
         if self._render_edit_stack.get_visible_child() == self._editor_scrolledwin:
             if self._headerbar_stack.get_visible_child() == self._search_header_bar:
                 self.__exit_search()
 
-            if not self.__render_view_added:
+            if self.__render_view is not None:
+                self.__continue_loading_webkit()
+            else:
                 self._render_edit_stack.set_visible_child(self._render_loading)
-
-            self.__render_view.render(
-                self.__note.content, self.__note.read_only, self.__get_editor_scroll_position()
-            )
-            self._search_header_bar.disable_actions()
+                # Push the WebKit initialisation down the road a little, giving the loading view
+                # time to initialise. A cleaner way to do this will be welcomed.
+                GLib.timeout_add(50, self.__continue_loading_webkit)
         else:
             if self._headerbar_stack.get_visible_child() == self._render_search_header_bar:
                 self.__exit_render_search()
@@ -893,14 +1032,17 @@ class Editor(Adw.Bin):
             self.__check_and_hide_headerbar_after_delay()
             self.focus_textview_if_editing()
             self._render_search_header_bar.disable_actions()
+            self.__focus_mode_action.set_enabled(True)
 
     def __check_and_terminate_webkit_process(self) -> None:
-        if not iotas.config_manager.get_markdown_keep_webkit_process():
+        if (
+            not iotas.config_manager.get_markdown_render_enabled()
+            or not iotas.config_manager.get_markdown_keep_webkit_process()
+        ):
+            logging.debug("Terminating WebKit process")
             self.__render_view.terminate_web_process()
             # Reduce flash when next loading render view
-            self.__render_view.hide()
-            self._render_edit_stack.remove(self.__render_view)
-            self.__render_view_added = False
+            self.__render_view.set_visible(False)
 
     def __category_changed(self, _obj: CategoryHeaderBar) -> None:
         changeset = self._category_header_bar.take_changeset()
@@ -937,7 +1079,8 @@ class Editor(Adw.Bin):
 
     def __refresh_line_length_from_setting(self) -> None:
         length = iotas.config_manager.get_line_length()
-        self._sourceview.set_property("maximum-width", length)
+        length_for_view = -1 if length == iotas.config_manager.get_line_length_max() else length
+        self._sourceview.line_length = length_for_view
         self._sourceview.queue_resize()
 
     def __desktop_setting_changed(
@@ -970,11 +1113,13 @@ class Editor(Adw.Bin):
             family = self.__font_families[font_setting]
             self._sourceview.update_font_family(family)
 
-        if iotas.config_manager.get_markdown_render_enabled():
+        if self.__render_view is not None:
             font_setting = self.__fetch_render_font_setting_name()
             if font_setting in self.__font_families:
                 family = self.__font_families[font_setting]
-                self.__render_view.update_font(family)
+                if self.__html_generator is not None:
+                    self.__html_generator.update_font_family(family)
+                self.__render_view.update_style()
 
     def __fetch_editor_font_setting_name(self):
         return (
@@ -1043,3 +1188,19 @@ class Editor(Adw.Bin):
         if self._toolbar_view.get_reveal_top_bars():
             self._toolbar_view.set_reveal_top_bars(False)
             self.focus_textview_if_editing()
+
+    def __ensure_webkit_initialised(self) -> None:
+        if self.__render_view is None:
+            self.__init_webkit()
+        return self.__render_view
+
+    def __continue_loading_webkit(self) -> None:
+        self.__ensure_webkit_initialised()
+
+        self.__render_view.render(
+            self.__note,
+            None,
+            self.__get_editor_scroll_position(),
+        )
+        self._search_header_bar.disable_actions()
+        self.__focus_mode_action.set_enabled(False)
