@@ -1,10 +1,13 @@
 from gettext import gettext as _
-from gi.repository import Gtk
+from gi.repository import GObject, Gtk
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Iotas/ui/editor_search_entry.ui")
 class EditorSearchEntry(Gtk.Box):
     __gtype_name__ = "EditorSearchEntry"
+    __gsignals__ = {
+        "activate": (GObject.SignalFlags.RUN_FIRST, None, ()),
+    }
 
     text = Gtk.Template.Child()
     _info = Gtk.Template.Child()
@@ -51,6 +54,10 @@ class EditorSearchEntry(Gtk.Box):
         """
         return self.text.set_text(text)
 
+    @Gtk.Template.Callback()
+    def _on_activate(self, _obj: GObject.Object) -> None:
+        self.emit("activate")
+
     def __update_position(self) -> None:
         if self.__occurrence_count == 0:
             occurrence_str = ""
