@@ -1,13 +1,14 @@
 import gi
 
 gi.require_version("GtkSource", "5")
-from gi.repository import Gdk, Gio, GLib, GObject, Gtk, GtkSource, Pango
+from gi.repository import Gdk, Gio, GLib, GObject, Gtk, GtkSource
 
 from gtkspellcheck import SpellChecker
 
 import locale
 import logging
 import re
+from typing import Callable, Optional
 
 import iotas.config_manager
 
@@ -15,8 +16,12 @@ import iotas.config_manager
 class EditorTextView(GtkSource.View):
     __gtype_name__ = "EditorTextView"
 
+    __gsignals__ = {
+        "size-allocated": (GObject.SignalFlags.RUN_FIRST, None, ()),
+    }
+
     # k is the token, v is the continuation
-    LIST_TOKENS = {
+    BULLET_LIST_TOKENS = {
         "- [ ] ": "- [ ] ",
         "- [x] ": "- [ ] ",
         "- ": "- ",
@@ -26,37 +31,21 @@ class EditorTextView(GtkSource.View):
 
     SOURCEVIEW_NO_SPELLCHECK_TAG = "gtksourceview:context-classes:no-spell-check"
 
+    BASE_MARGIN = 36
+    MINIMUM_MARGIN = 10
+
     def __init__(self):
         super().__init__()
 
         self.__css_provider = None
-        self.__maximum_width = -1
-        self.__font_family = "monospace"
+        self.__line_length = -1
         self.__longpress_gesture = None
         self.__longpress_popover = None
+        self.__font_family = "monospace"
 
-        # Use system setting for monospace font family
-        self.__dbus_proxy = Gio.DBusProxy.new_for_bus_sync(
-            bus_type=Gio.BusType.SESSION,
-            flags=Gio.DBusProxyFlags.NONE,
-            info=None,
-            name="org.freedesktop.portal.Desktop",
-            object_path="/org/freedesktop/portal/desktop",
-            interface_name="org.freedesktop.portal.Settings",
-            cancellable=None,
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.FONT_SIZE}", self.__on_font_size_changed
         )
-        if self.__dbus_proxy is None:
-            logging.warning("Unable to establish D-Bus proxy for FreeDesktop.org font setting")
-        else:
-            self.__load_font_family_from_setting(False)
-            self.__dbus_proxy.connect_object("g-signal", self.__desktop_setting_changed, None)
-
-        key = iotas.config_manager.FONT_SIZE
-        iotas.config_manager.settings.connect(f"changed::{key}", self.__on_font_size_changed)
-        key = iotas.config_manager.USE_MONOSPACE_FONT
-        if self.__dbus_proxy is not None:
-            iotas.config_manager.settings.connect(f"changed::{key}", self.__on_font_family_changed)
-        self.__update_font()
 
         controller = Gtk.EventControllerKey()
         controller.connect("key-pressed", self.__on_key_pressed)
@@ -66,8 +55,9 @@ class EditorTextView(GtkSource.View):
         if iotas.config_manager.get_spelling_enabled():
             self.__init_spellchecker()
 
-        key = iotas.config_manager.SPELLING_ENABLED
-        iotas.config_manager.settings.connect(f"changed::{key}", self.__on_spelling_toggled)
+        iotas.config_manager.settings.connect(
+            f"changed::{iotas.config_manager.SPELLING_ENABLED}", self.__on_spelling_toggled
+        )
 
         # TODO Temporary hack to provide access to spelling on mobile
         self.__setup_longpress_touch_menu()
@@ -82,38 +72,46 @@ class EditorTextView(GtkSource.View):
         """
         GtkSource.View.do_size_allocate(self, width, height, baseline)
 
-        # Update side margins
-        width = self.get_allocated_width()
-        if width < 1:
-            return
-        if self.__maximum_width > 0 and width > self.__maximum_width:
-            x_margin = (width - self.__maximum_width) / 2
-        else:
-            x_margin = 0
-        if self.get_left_margin() != x_margin:
-            self.set_left_margin(x_margin)
-            self.set_right_margin(x_margin)
-
         # Update top and bottom margin
         min_width_for_y_margin = 400
-        max_width_for_y_margin = self.__maximum_width if self.__maximum_width > 500 else 800
-        max_y_margin = 26
+        max_width_for_y_margin = self.__line_length if self.__line_length > 500 else 800
         if width <= min_width_for_y_margin:
-            y_margin = 0
+            y_margin = self.MINIMUM_MARGIN
         elif width < max_width_for_y_margin:
-            v_range = max_width_for_y_margin - min_width_for_y_margin
-            y_margin = int((width - min_width_for_y_margin) / v_range * float(max_y_margin))
+            value_range = max_width_for_y_margin - min_width_for_y_margin
+            y_range = float(self.BASE_MARGIN - self.MINIMUM_MARGIN)
+            in_range = width - min_width_for_y_margin
+            y_margin = self.MINIMUM_MARGIN + int(in_range / value_range * y_range)
         else:
-            y_margin = max_y_margin
-        y_margin += 10
+            y_margin = self.BASE_MARGIN
         if self.props.top_margin != y_margin:
             self.set_top_margin(y_margin)
             self.set_bottom_margin(y_margin)
 
-    def scroll_to_insertion_point(self) -> None:
-        """Scroll to the insertion point."""
+        # Update side margins
+        if width < 1:
+            return
+        if self.__line_length > 0 and width - 2 * self.BASE_MARGIN > self.__line_length:
+            x_margin = (width - self.__line_length) / 2
+        else:
+            x_margin = y_margin
+        if self.get_left_margin() != x_margin:
+            self.set_left_margin(x_margin)
+            self.set_right_margin(x_margin)
+
+        self.emit("size-allocated")
+
+    def jump_to_insertion_point(self) -> None:
+        """Jump to the insertion point.
+
+        This jump is used to avoid issues with height allocation in `scroll_to_mark` when the
+        buffer contains lines of varying height.
+        """
         buffer = self.get_buffer()
-        self.scroll_to_mark(buffer.get_insert(), 0.25, True, 1, 0.5)
+        insert_iter = buffer.get_iter_at_mark(buffer.get_insert())
+        location = self.get_iter_location(insert_iter)
+        adjustment = self.get_vadjustment()
+        adjustment.set_value(location.y)
 
     @GObject.Property(type=bool, default=False)
     def spellchecker_enabled(self) -> int:
@@ -126,19 +124,16 @@ class EditorTextView(GtkSource.View):
             self.__spellchecker.enabled = value
 
     @GObject.Property(type=int, default=-1)
-    def maximum_width(self) -> int:
-        return self.__maximum_width
+    def line_length(self) -> int:
+        return self.__line_length
 
-    @maximum_width.setter
-    def maximum_width(self, value: int) -> None:
-        self.__maximum_width = value
+    @line_length.setter
+    def line_length(self, value: int) -> None:
+        self.__line_length = value
 
     def __on_font_size_changed(self, _settings: Gio.Settings, _key: str) -> None:
         self.__update_font()
 
-    def __on_font_family_changed(self, _settings: Gio.Settings, _key: str) -> None:
-        self.__load_font_family_from_setting()
-
     # TODO Part of temporary hack to provide access to spelling menu on mobile
     def __on_longpress(self, _gesture: Gtk.GestureLongPress, x: float, y: float) -> None:
         # Long press menu is only currently used for spelling on mobile, don't show many if
@@ -169,16 +164,24 @@ class EditorTextView(GtkSource.View):
         keyval: int,
         keycode: int,
         state: Gdk.ModifierType,
-    ):
+    ) -> bool:
         if keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter, Gdk.KEY_ISO_Enter):
-            self.__process_newline()
-            return Gdk.EVENT_STOP
+            if self.__process_newline():
+                return Gdk.EVENT_STOP
         elif keyval in (Gdk.KEY_Tab, Gdk.KEY_ISO_Left_Tab):
             self.__handle_tab(state != Gdk.ModifierType.SHIFT_MASK)
             return Gdk.EVENT_STOP
 
         return Gdk.EVENT_PROPAGATE
 
+    def update_font_family(self, font_family: str) -> None:
+        """Update font family from system preference.
+
+        :param str font_family: The font family
+        """
+        self.__font_family = font_family
+        self.__update_font()
+
     def __update_font(self) -> None:
         if not self.__css_provider:
             self.__css_provider = Gtk.CssProvider()
@@ -247,57 +250,6 @@ class EditorTextView(GtkSource.View):
         logging.info(f'New spelling language "{language}"')
         iotas.config_manager.set_spelling_language(language)
 
-    def __fetch_font_setting_name(self):
-        return (
-            "monospace-font-name"
-            if iotas.config_manager.get_use_monospace_font()
-            else "document-font-name"
-        )
-
-    def __process_desktop_settings(self, variant: GLib.Variant) -> None:
-        if variant.get_type_string() != "(a{sa{sv}})":
-            return
-        setting_name = self.__fetch_font_setting_name()
-        for v in variant:
-            for key, value in v.items():
-                if key == "org.gnome.desktop.interface":
-                    if setting_name in value:
-                        self.__update_font_from_system_setting(value[setting_name])
-
-    def __desktop_setting_changed(
-        self, _sender_name: str, _signal_name: str, _parameters: str, data: GLib.Variant
-    ) -> None:
-        if _parameters != "SettingChanged":
-            return
-        (path, setting_name, value) = data
-        key_name = self.__fetch_font_setting_name()
-        if path == "org.gnome.desktop.interface" and setting_name == key_name:
-            if value.strip() != "":
-                self.__update_font_from_system_setting(value)
-
-    def __load_font_family_from_setting(self, apply_update: bool = True) -> None:
-        if self.__dbus_proxy is None:
-            return
-        try:
-            variant = self.__dbus_proxy.call_sync(
-                method_name="ReadAll",
-                parameters=GLib.Variant("(as)", ("org.gnome.desktop.*",)),
-                flags=Gio.DBusCallFlags.NO_AUTO_START,
-                timeout_msec=-1,
-                cancellable=None,
-            )
-        except GLib.GError as e:
-            logging.warning("Unable to access D-Bus FreeDesktop.org font family setting: %s", e)
-            return
-        self.__process_desktop_settings(variant)
-        if apply_update:
-            self.__update_font()
-
-    def __update_font_from_system_setting(self, font: str) -> None:
-        font_description = Pango.font_description_from_string(font)
-        self.__font_family = font_description.get_family()
-        self.__update_font()
-
     # TODO Part of temporary hack to provide access to spelling menu on mobile
     def __setup_longpress_touch_menu(self) -> None:
         gesture = Gtk.GestureLongPress.new()
@@ -310,53 +262,117 @@ class EditorTextView(GtkSource.View):
         self.__longpress_popover = Gtk.PopoverMenu.new_from_model(Gio.Menu())
         self.__longpress_popover.set_parent(self)
 
-    def __line_is_list_item(self, searchterm: str) -> re.Match:
+    def __check_line_for_bullet_list_item(self, searchterm: str) -> re.Match:
         term = r"^\s*("
         escaped_tokens = []
-        for token in self.LIST_TOKENS.keys():
+        for token in self.BULLET_LIST_TOKENS.keys():
             escaped_tokens.append(re.escape(token))
         term += "|".join(escaped_tokens)
         term += ")"
         return re.search(term, searchterm)
 
-    def __process_newline(self) -> None:
-        buffer = self.get_buffer()
-        buffer.insert_at_cursor("\n")
-        mark = buffer.get_insert()
-        location = buffer.get_iter_at_mark(mark)
-        line_start = location.copy()
-        line_start.backward_line()
-        line_start.set_line_offset(0)
-        prev_line = line_start.get_text(location)
+    def __check_line_for_ordered_list_item(self, searchterm: str) -> re.Match:
+        term = r"^(\s*)([a-zA-Z]{1}|[0-9]+)([\.\)]){1}[ ]+"
+        return re.search(term, searchterm)
 
-        # TODO look at changing to use the GtkSourceLanguage context
-        res = self.__line_is_list_item(prev_line)
-        if res is None:
-            return
-        matched_list_item = res.group()
+    def __process_newline(self) -> bool:
+        handled = False
+        buffer = self.get_buffer()
+        mark = buffer.get_insert()
+        insert_iter = buffer.get_iter_at_mark(mark)
+
+        line_start = insert_iter.copy()
+        line_start.set_line_offset(0)
+        previous_line = line_start.get_text(insert_iter)
+
+        # TODO look at changing to use the GtkSourceLanguage context?
+        regex_match = self.__check_line_for_bullet_list_item(previous_line)
+        if regex_match is not None:
+            handled = self.__extend_bullet_list(regex_match.group(), insert_iter, line_start)
+        else:
+            regex_match = self.__check_line_for_ordered_list_item(previous_line)
+            if regex_match is not None:
+                handled = self.__extend_ordered_list(regex_match.groups(), insert_iter, line_start)
+
+        if handled:
+            # Attempt to track cursor in viewport
+            mark = buffer.get_insert()
+            insert_iter = buffer.get_iter_at_mark(mark)
+            buffer_coords = self.get_iter_location(insert_iter)
+            visible_rect = self.get_visible_rect()
+            if buffer_coords.y + buffer_coords.height >= visible_rect.y + visible_rect.height:
+                # Without adding as an idle task we sometimes end up with an insufficient scroll
+                GLib.idle_add(self.emit, "move-viewport", Gtk.ScrollStep.STEPS, 1)
+
+        return handled
+
+    def __extend_bullet_list(
+        self, matched_list_item: str, insert_iter: Gtk.TextIter, line_start: Gtk.TextIter
+    ) -> bool:
+        buffer = self.get_buffer()
+        previous_line = line_start.get_text(insert_iter)
 
         # If the list already continues after our newline, don't add the next item markup.
         # Caters for accidentally deleting a line break in the middle of a list and then
         # pressing enter to revert that.
-        cur_line_end = location.copy()
+        cur_line_end = insert_iter.copy()
         if not cur_line_end.ends_line():
             cur_line_end.forward_to_line_end()
-        cur_line_text = location.get_text(cur_line_end)
-        if self.__line_is_list_item(cur_line_text) is not None:
-            return
+        cur_line_text = insert_iter.get_text(cur_line_end)
+        if self.__check_line_for_bullet_list_item(cur_line_text) is not None:
+            return False
+
+        def generate_sequence_previous_item() -> Optional[str]:
+            return matched_list_item
 
         empty_list_line = self.inserted_empty_item_at_end_of_list(
-            prev_line, matched_list_item, line_start
+            previous_line, matched_list_item, line_start, generate_sequence_previous_item
         )
         if empty_list_line:
             # An empty list line has been entered, remove it from the list end
-            buffer.delete(line_start, location)
+            buffer.delete(line_start, insert_iter)
             buffer.insert_at_cursor("\n")
         else:
             dict_key = matched_list_item.lstrip()
             spacing = matched_list_item[0 : len(matched_list_item) - len(dict_key)]
-            new_entry = f"{spacing}{self.LIST_TOKENS[dict_key]}"
-            buffer.insert(location, new_entry)
+            new_entry = f"\n{spacing}{self.BULLET_LIST_TOKENS[dict_key]}"
+            buffer.insert_at_cursor(new_entry)
+        return True
+
+    def __extend_ordered_list(
+        self, regex_groups: tuple, insert_iter: Gtk.TextIter, line_start: Gtk.TextIter
+    ) -> bool:
+        buffer = self.get_buffer()
+        spacing = regex_groups[0]
+        index = regex_groups[1]
+        marker = regex_groups[2]
+        previous_line = line_start.get_text(insert_iter)
+
+        def generate_sequence_previous_item() -> Optional[str]:
+            # Verify there's more than one list item. This allows inserting lines with list start
+            # tokens and nothing else.
+            previous_index = EditorTextView.calculate_ordered_list_index(index, -1)
+            if previous_index is None:
+                return None
+            else:
+                return self.format_ordered_list_item(spacing, previous_index, marker)
+
+        empty_list_line = self.inserted_empty_item_at_end_of_list(
+            previous_line, "".join(regex_groups), line_start, generate_sequence_previous_item
+        )
+        if empty_list_line:
+            # An empty list line has been entered, remove it from the list end
+            buffer.delete(line_start, insert_iter)
+            buffer.insert_at_cursor("\n")
+        else:
+            sequence_next = self.calculate_ordered_list_index(index, 1)
+            # Handle value of Z, ending sequence
+            if sequence_next is None:
+                return False
+            new_entry = "\n" + self.format_ordered_list_item(spacing, sequence_next, marker)
+            buffer.insert(insert_iter, new_entry)
+
+        return True
 
     def __handle_tab(self, increase: bool) -> None:
         buffer = self.get_buffer()
@@ -399,7 +415,10 @@ class EditorTextView(GtkSource.View):
         if multi_line and line_contents == "":
             return
 
-        list_item = self.__line_is_list_item(line_contents) is not None
+        list_item = (
+            self.__check_line_for_bullet_list_item(line_contents) is not None
+            or self.__check_line_for_ordered_list_item(line_contents) is not None
+        )
         indent_chars = "  " if list_item else "\t"
 
         if increase:
@@ -415,16 +434,71 @@ class EditorTextView(GtkSource.View):
 
     @staticmethod
     def inserted_empty_item_at_end_of_list(
-        line_text: str, matched_list_item: str, previous_line_start: Gtk.TextIter
+        line_text: str,
+        matched_list_item: str,
+        previous_line_start: Gtk.TextIter,
+        fetch_sequence_previous: Callable[[], Optional[str]],
     ) -> bool:
-        # Check if previous line contains only the list item markup
+        """Determine whether we have inserted an empty item at the end of a list
+
+        The list needs to have at least one previous item.
+
+        :param str line_text: The full line text
+        :param str matched_list_item: Our matched list marker
+        :param Gtk.TextIter previous_line_start: The start of the previous line
+        :param Callable[], Optional[str]] fetch_sequence_previous: A function to fetch a previous
+            line's marker
+        :return: Whether we've inserted an empty line
+        :rtype: bool
+        """
+
+        # Check if entered line contains only the list item markup
         if line_text.strip() != matched_list_item.strip():
             return False
 
+        # Fetch what a previous item in the sequence would have been, for the check below. The
+        # callable is used to prevent calculating this before we even know if match above will
+        # pass (while sharing logic for bullet and ordered lists).
+        sequence_previous_item = fetch_sequence_previous()
+        if sequence_previous_item is None:
+            return False
+
         # Verify there's more than one list item. This allows inserting lines with list start
         # tokens and nothing else.
         two_prev_start = previous_line_start.copy()
         two_prev_start.backward_line()
         two_prev_start.set_line_offset(0)
         two_prev_line = two_prev_start.get_text(previous_line_start)
-        return two_prev_line.startswith(matched_list_item)
+        return two_prev_line.startswith(sequence_previous_item)
+
+    @staticmethod
+    def calculate_ordered_list_index(reference: str, direction: int) -> Optional[str]:
+        """Calculate an item in an ordered list sequence.
+
+        :param str reference: The element to start from
+        :param str direction: The direction, 1 or -1
+        :return: The item
+        :rtype: Optional[str]
+        """
+        sequence_next = None
+        if reference.isdigit():
+            sequence_next = str(int(reference) + direction)
+            if sequence_next == 0:
+                sequence_next = None
+        elif direction > 0 and reference.upper() != "Z":
+            sequence_next = chr(ord(reference) + 1)
+        elif direction < 0 and reference.upper() != "A":
+            sequence_next = chr(ord(reference) - 1)
+        return sequence_next
+
+    @staticmethod
+    def format_ordered_list_item(spacing: str, index: str, delimiter: str) -> str:
+        """Generate string for list item.
+
+        :param str spacing: Pre marker spacing
+        :param str index: Item index
+        :param str delimiter: The delimiter before the item text
+        :return: The build string
+        :rtype: str
+        """
+        return f"{spacing}{index}{delimiter} "
