@@ -1,4 +1,5 @@
 from gettext import gettext as _
+from gettext import ngettext
 from gi.repository import Adw, Gdk, GObject, Gio, GLib, Gtk
 
 import logging
@@ -8,6 +9,7 @@ from iotas.category import Category, CategorySpecialPurpose
 from iotas.category_manager import CategoryManager
 import iotas.config_manager
 from iotas import const
+from iotas.index_search_header_bar import IndexSearchState
 from iotas.note import Note
 from iotas.note_manager import NoteManager
 from iotas.selection_header_bar import SelectionHeaderBar
@@ -28,6 +30,7 @@ class Index(Adw.BreakpointBin):
     UNDO_NOTE_DELETION_DURATION = 5.0
     SYNC_CONFLICT_NOTIFICATION_DURATION = 5.0
     SYNC_UPDATE_NOTIFICATION_DURATION = 2.0
+    NOTES_APP_MISSING_NOTIFICATION_DURATION = 8.0
     BREAKPOINT_WIDTH = 700
 
     _split_view = Gtk.Template.Child()
@@ -66,12 +69,13 @@ class Index(Adw.BreakpointBin):
         self.__category_manager = None
         self.__sync_manager = None
 
+        self.__active = False
         self.__add_fav_action = None
         self.__remove_fav_action = None
         self.__initialised = False
         self.__search_after_init = None
         self.__searching_before_selection = False
-        self.__active = False
+        self.__cancel_action = None
 
         # Startup bootstrap category
         self.__category = Category("", 0)
@@ -144,6 +148,7 @@ class Index(Adw.BreakpointBin):
             self.__category_manager,
             self.__on_category_activated,
         )
+        self._search_header_bar.setup(self._note_list, self.__note_manager)
         self._selection_header_bar.setup(self.__category_manager.tree_store)
         self._note_list.setup(self.__note_manager, self.__on_listbox_key_pressed)
         self._selection_header_bar.bind_property(
@@ -162,6 +167,9 @@ class Index(Adw.BreakpointBin):
         self.__sync_manager.connect("started", self.__on_sync_started)
         self.__sync_manager.connect("finished", self.__on_sync_finished)
         self.__sync_manager.connect("missing-password", self.__on_missing_password)
+        self.__sync_manager.connect(
+            "notes-capability-missing", self.__on_missing_server_notes_capability
+        )
 
         self.__sync_manager.init_auth()
 
@@ -194,14 +202,31 @@ class Index(Adw.BreakpointBin):
             self.__invalidate_view()
             self.__handle_emptied_category()
 
-    def update_layout_for_initial_width(self, width: int) -> None:
-        """Update layout for initial window width.
+    def update_for_dialog_visibility(self, visible: bool) -> None:
+        """Update if dialogs visible, disabling actions.
+
+        :param bool visible: Whether a dialog is visible
+        """
+        if self.active:
+            self.__cancel_action.set_enabled(not visible)
+
+    def update_layout_for_initial_size(self, width: int, height: int) -> None:
+        """Update layout for initial window size.
 
         :param int width: Window width
+        :param int height: Window height
         """
         self.__update_for_sidebar_persistence_change(width)
         if not iotas.config_manager.get_persist_sidebar():
             self._split_view.set_collapsed(True)
+        self._note_list.update_search_pagesize_for_height(height)
+
+    def update_search_pagesize_for_height(self, height: int) -> None:
+        """Recalculate number of results to show on first search page.
+
+        :param int height: The new window height
+        """
+        self._note_list.update_search_pagesize_for_height(height)
 
     def show_note_conflict_alert(self) -> None:
         """Let the user know a sync conflict has occurred with the note being edited."""
@@ -227,7 +252,7 @@ class Index(Adw.BreakpointBin):
         )
 
     def refresh_after_note_closed(self, note: Note) -> None:
-        """Refresh index afer note has been closed.
+        """Refresh index after note has been closed.
 
         :param Note note: The closed note
         """
@@ -267,6 +292,15 @@ class Index(Adw.BreakpointBin):
     def _on_reauthenticate(self, _button: Gtk.Button) -> None:
         self.emit("reauthenticate")
 
+    @Gtk.Template.Callback()
+    def _on_edge_overshot(self, _obj: Gtk.ScrolledWindow, pos: Gtk.PositionType) -> None:
+        if pos == Gtk.PositionType.BOTTOM:
+            if not self.__searching() and not self._note_list.older_notes_displayed:
+                self.__load_older_notes()
+            elif self.__searching() and self._note_list.have_more_search_results:
+                self._note_list.show_remaining_search_results()
+                self.__note_manager.update_filters(self.__category)
+
     def __on_search_changed(self, _object: GObject.Object) -> None:
         self.__apply_search()
 
@@ -299,7 +333,11 @@ class Index(Adw.BreakpointBin):
         app.set_accels_for_action("index.delete-from-index", ["<Shift>Delete"])
 
         action = Gio.SimpleAction.new("load-older-notes")
-        action.connect("activate", self.__load_older_notes)
+        action.connect("activate", self.__on_load_older_notes)
+        action_group.add_action(action)
+
+        action = Gio.SimpleAction.new("show-more")
+        action.connect("activate", self.__on_show_more)
         action_group.add_action(action)
 
         action = Gio.SimpleAction.new("enter-search")
@@ -315,6 +353,7 @@ class Index(Adw.BreakpointBin):
         action = Gio.SimpleAction.new("cancel")
         action.connect("activate", self.__cancel)
         action_group.add_action(action)
+        self.__cancel_action = action
         app.set_accels_for_action("index.cancel", ["Escape", "<Alt>Left"])
 
         action = Gio.SimpleAction.new("reset-filters")
@@ -327,6 +366,11 @@ class Index(Adw.BreakpointBin):
         action_group.add_action(action)
         app.set_accels_for_action("index.selection-toggle-favourite", ["<Control>g"])
 
+        action = Gio.SimpleAction.new("show-menu")
+        action.connect("activate", self.__on_show_menu)
+        action_group.add_action(action)
+        app.set_accels_for_action("index.show-menu", ["F10"])
+
         self.__action_group = action_group
         app.get_active_window().insert_action_group("index", action_group)
 
@@ -339,10 +383,12 @@ class Index(Adw.BreakpointBin):
         for action in actions:
             self.__action_group.lookup_action(action).set_enabled(enabled)
 
-    def __load_older_notes(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        self.__load_older_notes_and_notify()
-        self.__invalidate_view()
-        self._note_list.update_category_labels(self.__category)
+    def __on_load_older_notes(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        self.__load_older_notes()
+
+    def __on_show_more(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        self._note_list.show_remaining_search_results()
+        self.__note_manager.update_filters(self.__category)
 
     def __on_realize(self, _widget: Gtk.Widget) -> None:
         self.__note_manager.update_filters(self.__category)
@@ -435,7 +481,7 @@ class Index(Adw.BreakpointBin):
             self._note_list.move_focus_to_list_top()
 
     def __on_enter_search(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
-        self.__enter_search(True)
+        self.__enter_search(clear_text=True)
 
     def __on_enter_selection(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
         self.__start_selection()
@@ -460,12 +506,8 @@ class Index(Adw.BreakpointBin):
                 self.__invalidate_view()
                 self._note_list.update_category_labels(self.__category)
 
-                if changes == 1:
-                    # Translators: Description, notification
-                    msg = _("1 change")
-                else:
-                    # Translators: Description, notification, {} is a number larger than 1
-                    msg = _("{} changes").format(changes)
+                # Translators: Description, notification, {} is a number
+                msg = ngettext("{} change", "{} changes", changes).format(changes)
                 self._timed_notification.show(
                     msg, self.SYNC_UPDATE_NOTIFICATION_DURATION, show_icon=True
                 )
@@ -479,6 +521,13 @@ class Index(Adw.BreakpointBin):
             else:
                 self._generic_reauthenticate_banner.set_revealed(True)
 
+    def __on_missing_server_notes_capability(self, _obj: GObject.Object) -> None:
+        self._timed_notification.show(
+            # Translators: Description, notification
+            _("Sync failure. Is the Nextcloud Notes app installed on the server?"),
+            duration=self.NOTES_APP_MISSING_NOTIFICATION_DURATION,
+        )
+
     def __on_index_key_pressed(
         self,
         controller: Gtk.EventControllerKey,
@@ -501,12 +550,14 @@ class Index(Adw.BreakpointBin):
             self._note_list.move_focus_to_list_top()
             return Gdk.EVENT_STOP
         elif not self.__searching() and not self.__selecting():
-            result = self._search_header_bar.check_if_starting(controller, keyval, state)
-            if result:
-                self.__enter_search(False)
-            return result
-        elif self.__searching():
-            if self._search_header_bar.check_for_open_first_result_shortcut(
+            if self._search_header_bar.check_if_starting(controller, keyval, state):
+                self.__enter_search(clear_text=False)
+                return Gdk.EVENT_STOP
+        elif self.__searching() and not self.__selecting():
+            if self._search_header_bar.check_if_starting(controller, keyval, state, clear=False):
+                self._search_header_bar.focus_entry()
+                return Gdk.EVENT_STOP
+            elif self._search_header_bar.check_for_open_first_result_shortcut(
                 controller, keyval, state
             ):
                 # Check that we have some results, a litle indirect
@@ -530,10 +581,13 @@ class Index(Adw.BreakpointBin):
             self._note_list.focus_previous_list_row(self.__get_focused())
             return Gdk.EVENT_STOP
         elif not self.__searching() and not self.__selecting():
-            result = self._search_header_bar.check_if_starting(controller, keyval, state)
-            if result == Gdk.EVENT_STOP:
-                self.__enter_search(False)
-            return result
+            if self._search_header_bar.check_if_starting(controller, keyval, state):
+                self.__enter_search(clear_text=False)
+                return Gdk.EVENT_STOP
+        elif self.__searching() and not self.__selecting():
+            if self._search_header_bar.check_if_starting(controller, keyval, state, clear=False):
+                self._search_header_bar.focus_entry()
+                return Gdk.EVENT_STOP
 
         return Gdk.EVENT_PROPAGATE
 
@@ -555,6 +609,9 @@ class Index(Adw.BreakpointBin):
         if self.__selecting():
             self._selection_header_bar.toggle_favourite_on_selection()
 
+    def __on_show_menu(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
+        self._menu_button.popup()
+
     def __on_category_shortcut(self, _action: Gio.SimpleAction, _param: GLib.Variant) -> None:
         if self.__selecting():
             self._selection_header_bar.edit_category_for_selection()
@@ -575,6 +632,14 @@ class Index(Adw.BreakpointBin):
             self._split_view.set_collapsed(True)
         self.__update_header_buttons_for_state()
 
+    def __load_older_notes(self) -> None:
+        if not self._note_list.older_notes_loaded:
+            self.__load_older_notes_and_notify()
+        else:
+            self._note_list.display_older_notes()
+        self.__invalidate_view()
+        self._note_list.update_category_labels(self.__category)
+
     def __create_and_open_note(self, content: Optional[str] = None) -> None:
         note = self.__note_manager.create_note(self.__category)
         # If we're creating a note from a selection persist it straight away
@@ -607,7 +672,7 @@ class Index(Adw.BreakpointBin):
 
     def __start_search_from_cli(self, search_term: str) -> None:
         self._search_header_bar.text = search_term
-        self.__enter_search(False, True)
+        self.__enter_search(clear_text=False, avoid_select_all=True)
 
     def __enter_search(self, clear_text, avoid_select_all: bool = False) -> None:
         """Enter the search UI."""
@@ -631,23 +696,22 @@ class Index(Adw.BreakpointBin):
     def __apply_search(self) -> None:
         if not self.__searching():
             return
+
         # If trying to search without caching completed delay the search
         if not self._note_list.older_notes_loaded:
             GLib.timeout_add(500, self.__apply_search)
             logging.debug("Delaying search due to model not ready")
             return
-        search_text = self._search_header_bar.text
-        if len(search_text) > 1:
-            found_ids = self.__note_manager.search_notes(search_text)
-            if len(found_ids) > 0:
-                self._note_list.restrict_for_search_by_ids(found_ids)
-                self.__invalidate_view()
-                self._note_list.update_category_labels(self.__category)
-                self._stack.set_visible_child(self._note_list_scroll)
-                self._note_list_scroll.get_vadjustment().set_value(0)
-            else:
-                self._stack.set_visible_child(self._search_empty)
-        else:
+
+        result = self._search_header_bar.search()
+        if result == IndexSearchState.RESULTS:
+            self.__invalidate_view()
+            self._note_list.update_category_labels(self.__category)
+            self._stack.set_visible_child(self._note_list_scroll)
+            self._note_list_scroll.get_vadjustment().set_value(0)
+        elif result == IndexSearchState.EMPTY:
+            self._stack.set_visible_child(self._search_empty)
+        elif result == IndexSearchState.INFO:
             self._stack.set_visible_child(self._search_info)
             self._note_list.restrict_for_search_by_ids(None)
 
@@ -704,8 +768,14 @@ class Index(Adw.BreakpointBin):
         self._note_list_scroll.get_vadjustment().set_value(0)
         self.__set_search_visible(False)
         self.__on_category_activated(self.__category)
-        if note is not None and self.get_root().using_keyboard_navigation:
-            self._note_list.select_and_focus_note(note)
+        if self.get_root().using_keyboard_navigation:
+            if note is not None:
+                self._note_list.select_and_focus_note(note)
+            else:
+                self._note_list.move_focus_to_list_top()
+        else:
+            # Prevent focus loss, type to search stopping to work
+            self._content_sidebar_button.grab_focus()
 
     def __update_enabled_actions(self, enable: bool) -> None:
         for action in ("toggle-sidebar", "create-note", "enter-search", "select-notes"):
@@ -751,6 +821,7 @@ class Index(Adw.BreakpointBin):
         self.__older_notes_model = self.__note_manager.initiate_older_notes_model()
         if time_filtered:
             self.__note_manager.filter_older_notes_by_date()
+
         loaded_notes = self._note_list.populate_older_notes(
             self.__note_manager.older_notes_model, self.__on_listbox_key_pressed, displayed
         )
