@@ -19,6 +19,9 @@ class IndexNoteList(Gtk.Box):
         "note-checkbox-deactivated": (GObject.SignalFlags.RUN_FIRST, None, (Note,)),
     }
 
+    MINIMUM_SEARCH_PAGE_NOTE_COUNT = 15
+    ROW_HEIGHT = 80
+
     _favourites_listbox = Gtk.Template.Child()
     _favourites_section = Gtk.Template.Child()
     _today_listbox = Gtk.Template.Child()
@@ -35,13 +38,16 @@ class IndexNoteList(Gtk.Box):
     _older_notes_listbox = Gtk.Template.Child()
     _older_notes_section = Gtk.Template.Child()
     _older_notes_label = Gtk.Template.Child()
+    _show_more_button = Gtk.Template.Child()
 
     def __init__(self):
         super().__init__()
 
         self.__selecting = False
         self.__searching = False
+        self.__search_full_set = []
         self.__older_notes_displayed = False
+        self.__search_base_note_count = self.MINIMUM_SEARCH_PAGE_NOTE_COUNT
 
         self.__listboxes = []
         self.__listboxes.append(self._favourites_listbox)
@@ -111,6 +117,15 @@ class IndexNoteList(Gtk.Box):
             listbox.connect("row-activated", self.__on_row_activated)
             listbox.connect("row-selected", self.__on_row_selected)
 
+    def update_search_pagesize_for_height(self, height: int) -> None:
+        """Recalculate number of results to show on first search "page".
+
+        :param int height: The new window height
+        """
+        self.__search_base_note_count = max(
+            self.MINIMUM_SEARCH_PAGE_NOTE_COUNT, int((height / self.ROW_HEIGHT) + 5)
+        )
+
     def clear_selections(self) -> None:
         """Clear all selections in the list."""
         self.__clear_selections_excluding()
@@ -152,9 +167,11 @@ class IndexNoteList(Gtk.Box):
             self._older_notes_listbox.connect("row-activated", self.__on_row_activated)
             self._older_notes_listbox.connect("row-selected", self.__on_row_selected)
 
-        self._older_notes_loader.set_visible(False)
+        if display_section:
+            self._older_notes_loader.set_visible(False)
         if len(self.__older_notes_model) > 0:
-            self._older_notes_section.set_visible(True)
+            if display_section:
+                self._older_notes_section.set_visible(True)
             loaded_notes = True
 
         return loaded_notes
@@ -268,7 +285,7 @@ class IndexNoteList(Gtk.Box):
                 index_row.update_category(current_category, style)
                 child = child.get_next_sibling()
 
-    def select_and_focus_note(self, note: Note):
+    def select_and_focus_note(self, note: Note) -> None:
         """Attempt to select and focus the provided note.
 
         :param Note note: The note
@@ -322,8 +339,21 @@ class IndexNoteList(Gtk.Box):
 
         :param Optional[List[int]] ids: Identifiers
         """
+        if ids and len(ids) > self.__search_base_note_count:
+            self.__search_full_set = ids
+            self._show_more_button.set_visible(True)
+            ids = ids[: self.__search_base_note_count]
+        else:
+            self._show_more_button.set_visible(False)
         self.__older_notes_model.restrict_by_id(ids)
 
+    def show_remaining_search_results(self) -> None:
+        """Show remaining search results below first "page"."""
+        if len(self.__search_full_set) > 0:
+            self.__older_notes_model.restrict_by_id(self.__search_full_set)
+            self.__search_full_set = []
+            self._show_more_button.set_visible(False)
+
     def open_first_search_result(self) -> None:
         """Open the top search result."""
         listbox = self._older_notes_listbox
@@ -336,6 +366,17 @@ class IndexNoteList(Gtk.Box):
             note = row.get_child().note
             self.emit("note-opened", note)
 
+    def display_older_notes(self) -> None:
+        """Display notes older than two months."""
+        if self.older_notes_displayed:
+            return
+        elif not self.older_notes_loaded:
+            return
+        self.older_notes_displayed = True
+        self._older_notes_loader.set_visible(False)
+        if len(self.__older_notes_model) > 0:
+            self._older_notes_section.set_visible(True)
+
     @GObject.Property(type=bool, default=False)
     def older_notes_displayed(self) -> bool:
         return self.__older_notes_displayed
@@ -368,6 +409,10 @@ class IndexNoteList(Gtk.Box):
     def selecting(self, value: bool) -> None:
         self.__selecting = value
 
+    @GObject.Property(type=bool, default=False)
+    def have_more_search_results(self) -> bool:
+        return len(self.__search_full_set) > 0
+
     def __create_row(self, note: Note, _user_data: Any) -> Gtk.Widget:
         row = IndexRow()
         style = iotas.config_manager.get_index_category_style()
@@ -424,8 +469,8 @@ class IndexNoteList(Gtk.Box):
             self.emit("note-opened", note)
             self.__clear_selections_excluding(listbox)
 
-    def __on_row_selected(self, listbox: Gtk.ListBox, row: Gtk.ListBoxRow) -> None:
-        if self.selecting:
+    def __on_row_selected(self, listbox: Gtk.ListBox, row: Optional[Gtk.ListBoxRow]) -> None:
+        if row:
             self.__clear_selections_excluding(listbox)
 
     def __on_row_context_click(self, row: IndexRow) -> None:
