@@ -7,7 +7,6 @@ import re
 from iotas.category import Category
 import iotas.config_manager
 from iotas.note import Note
-from iotas.ui_utils import add_mouse_right_click_accel, add_touch_longpress_accel
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Iotas/ui/index_row.ui")
@@ -30,11 +29,6 @@ class IndexRow(Gtk.Box):
         self.__note = None
         self.__signal_ids = list()
         self.__index_category = None
-
-        add_touch_longpress_accel(
-            self, self.__start_selection, propagation=Gtk.PropagationPhase.CAPTURE
-        )
-        add_mouse_right_click_accel(self, self.__start_selection)
         self.connect("unrealize", self.__on_unrealize)
 
     def populate(self, note: Note, category_style: str) -> None:
@@ -99,6 +93,16 @@ class IndexRow(Gtk.Box):
     def _on_selection_checkbox_toggled(self, button: Gtk.CheckButton) -> None:
         self.emit("checkbox-toggled", button.get_active())
 
+    @Gtk.Template.Callback()
+    def _on_longpress(self, _gesture: Gtk.GestureLongPress, _x: float, _y: float) -> None:
+        self.emit("context-click")
+
+    @Gtk.Template.Callback()
+    def _on_right_click(
+        self, _gesture: Gtk.GestureClick, _n_press: int, _x: float, _y: float
+    ) -> None:
+        self.emit("context-click")
+
     def __on_title_or_dirty_changed(
         self,
         _obj: GObject.Object,
@@ -113,9 +117,6 @@ class IndexRow(Gtk.Box):
     ) -> None:
         self.__refresh_category_label()
 
-    def __start_selection(self) -> None:
-        self.emit("context-click")
-
     def __refresh_title_label(self) -> None:
         txt = self.__note.title
         remote_syncing = iotas.config_manager.nextcloud_sync_configured()
