@@ -1,8 +1,18 @@
 from gi.repository import Adw, Gdk, GObject, Gtk
 
+from enum import IntEnum, auto
+
+from iotas.index_note_list import IndexNoteList
+from iotas.note_manager import NoteManager
 from iotas.ui_utils import check_for_search_starting
 
 
+class IndexSearchState(IntEnum):
+    INFO = auto()
+    EMPTY = auto()
+    RESULTS = auto()
+
+
 @Gtk.Template(resource_path="/org/gnome/World/Iotas/ui/index_search_header_bar.ui")
 class IndexSearchHeaderBar(Adw.Bin):
     __gtype_name__ = "IndexSearchHeaderBar"
@@ -16,21 +26,36 @@ class IndexSearchHeaderBar(Adw.Bin):
         super().__init__()
         self.__active = False
 
+    def setup(self, note_list: IndexNoteList, note_manager: NoteManager) -> None:
+        """Perform initial setup.
+
+        :param IndexNoteList note_list: The note list
+        :param NoteManager note_manager: The note manager
+        """
+        self.__note_list = note_list
+        self.__note_manager = note_manager
+
     def check_if_starting(
-        self, controller: Gtk.EventControllerKey, keyval: int, state: Gdk.ModifierType
+        self,
+        controller: Gtk.EventControllerKey,
+        keyval: int,
+        state: Gdk.ModifierType,
+        clear: bool = True,
     ) -> bool:
         """Check if starting search via type to search.
 
         :param Gtk.EventControllerKey controller: The key controller
         :param int keyval: The key value
         :param Gdk.ModifierType state: Any modifier state
+        :param bool clear: Whether to clear the entry
         :return: Whether search is starting
         :rtype: bool
         """
         if not check_for_search_starting(controller, keyval, state):
             return False
 
-        self._entry.set_text("")
+        if clear:
+            self._entry.set_text("")
         if controller.forward(self._entry.get_delegate()):
             return True
 
@@ -68,6 +93,23 @@ class IndexSearchHeaderBar(Adw.Bin):
             self._entry.select_region(0, -1)
         self.active = True
 
+    def search(self) -> IndexSearchState:
+        """Perform a search.
+
+        :return: The resulting search state
+        :rtype: IndexSearchState
+        """
+        search_text = self.text
+        if len(search_text) > 0:
+            found_ids = self.__note_manager.search_notes(search_text, sort=True)
+            if len(found_ids) > 0:
+                self.__note_list.restrict_for_search_by_ids(found_ids)
+                return IndexSearchState.RESULTS
+            else:
+                return IndexSearchState.EMPTY
+        else:
+            return IndexSearchState.INFO
+
     def exit(self) -> None:
         """Exit search.
 
@@ -75,6 +117,10 @@ class IndexSearchHeaderBar(Adw.Bin):
         """
         self.active = False
 
+    def focus_entry(self) -> None:
+        """Focus the text entry."""
+        self._entry.grab_focus()
+
     @GObject.Property(type=bool, default=False)
     def active(self) -> bool:
         return self.__active
