@@ -1,7 +1,7 @@
 import gi
 
 gi.require_version("Secret", "1")
-from gi.repository import Gdk, GObject, GLib, Gtk, Secret
+from gi.repository import Gdk, Gio, GObject, GLib, Gtk, Secret
 
 from email.utils import parsedate_to_datetime
 from json.decoder import JSONDecodeError
@@ -22,6 +22,7 @@ class LoginRequestResult:
     SUCCESS = 0
     FAILURE_GENERIC = 1
     FAILURE_SELF_SIGNED_SSL = 2
+    FAILURE_CERTIFICATE = 3
 
 
 class NextcloudSyncWorker(GObject.Object):
@@ -31,6 +32,7 @@ class NextcloudSyncWorker(GObject.Object):
         "settings-fetched": (GObject.SignalFlags.RUN_FIRST, None, (str,)),
         "secret-service-failure": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "missing-password": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "notes-capability-missing": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     ENDPOINT_V_1_0 = "/index.php/apps/notes/api/v1/"
@@ -63,6 +65,13 @@ class NextcloudSyncWorker(GObject.Object):
         self.__username = iotas.config_manager.get_nextcloud_username()
         self.__prune_threshold = iotas.config_manager.get_nextcloud_prune_threshold()
 
+        iotas.config_manager.settings.bind(
+            iotas.config_manager.NETWORK_TIMEOUT,
+            self,
+            "timeout",
+            Gio.SettingsBindFlags.DEFAULT,
+        )
+
         if os.path.exists(self.CA_CHAIN_FILE):
             logging.info("Using SSL CA chain file at {}".format(self.CA_CHAIN_FILE))
             self.verify = self.CA_CHAIN_FILE
@@ -90,11 +99,21 @@ class NextcloudSyncWorker(GObject.Object):
         """
         uri = "{}{}".format(server_uri, self.LOGIN_ENDPOINT)
         try:
-            res = requests.post(uri, headers=self.__headers, verify=self.verify)
+            res = requests.post(
+                uri,
+                headers=self.__headers,
+                verify=self.verify,
+                timeout=self.__timeout,
+            )
         except requests.exceptions.SSLError as e:
             if self.exception_is_for_self_signed_certificate(e):
                 logging.warning("Login found self-signed certificate: {} for {}".format(e, uri))
                 return (LoginRequestResult.FAILURE_SELF_SIGNED_SSL, None, None)
+            elif self.get_ssl_verification_error(e) is not None:
+                logging.warning(
+                    "Login POST encountered SSL certificate problem: {} for {}".format(e, uri)
+                )
+                return (LoginRequestResult.FAILURE_CERTIFICATE, None, None)
             else:
                 logging.warning("Login POST request error: {} for {}".format(e, uri))
                 return (LoginRequestResult.FAILURE_GENERIC, None, None)
@@ -134,7 +153,13 @@ class NextcloudSyncWorker(GObject.Object):
         """
         data = {"token": token}
         try:
-            res = requests.post(endpoint, headers=self.__headers, data=data, verify=self.verify)
+            res = requests.post(
+                endpoint,
+                headers=self.__headers,
+                data=data,
+                verify=self.verify,
+                timeout=self.__timeout,
+            )
         except Exception:
             return (False, False)
         if res.status_code != 200:
@@ -171,6 +196,7 @@ class NextcloudSyncWorker(GObject.Object):
                 headers=headers,
                 auth=(self.__username, self.__app_password),
                 verify=self.verify,
+                timeout=self.__timeout,
             )
         except Exception as e:
             logging.warning("GET request for capabilities failed: %s", e)
@@ -188,6 +214,7 @@ class NextcloudSyncWorker(GObject.Object):
                 "Couldn't access version elements in server capabilities response. "
                 "Is the Nextcloud Notes app installed within Nextcloud?"
             )
+            GLib.idle_add(self.emit, "notes-capability-missing")
             return
 
         logging.info(
@@ -313,6 +340,14 @@ class NextcloudSyncWorker(GObject.Object):
     def secret_service_failed(self) -> bool:
         return self.__secret_service_failed
 
+    @GObject.Property(type=int)
+    def timeout(self) -> int:
+        return self.__timeout
+
+    @timeout.setter
+    def timeout(self, value: int) -> None:
+        self.__timeout = value
+
     def __init_secrets(self) -> None:
         # Doing this sync fetch works around some type of libsecret flatpak issue (... so we may
         # as well use the result)
@@ -356,6 +391,7 @@ class NextcloudSyncWorker(GObject.Object):
                 auth=(self.__username, self.__app_password),
                 params=params,
                 verify=self.verify,
+                timeout=self.__timeout,
             )
         except Exception as e:
             logging.warning("GET request failed: %s", e)
@@ -370,6 +406,7 @@ class NextcloudSyncWorker(GObject.Object):
                 auth=(self.__username, self.__app_password),
                 data=data,
                 verify=self.verify,
+                timeout=self.__timeout,
             )
         except Exception as e:
             logging.warning("PUT request failed: %s", e)
@@ -384,6 +421,7 @@ class NextcloudSyncWorker(GObject.Object):
                 auth=(self.__username, self.__app_password),
                 data=data,
                 verify=self.verify,
+                timeout=self.__timeout,
             )
         except Exception as e:
             logging.warning("POST request failed: %s", e)
@@ -397,6 +435,7 @@ class NextcloudSyncWorker(GObject.Object):
                 headers=self.__headers,
                 auth=(self.__username, self.__app_password),
                 verify=self.verify,
+                timeout=self.__timeout,
             )
         except Exception as e:
             logging.warning("DELETE request failed: %s", e)
@@ -478,20 +517,28 @@ class NextcloudSyncWorker(GObject.Object):
 
     @staticmethod
     def exception_is_for_self_signed_certificate(e: requests.exceptions.SSLError) -> bool:
-        # Get the requests SSLError
-        if not type(e.args) is tuple or len(e.args) != 1:
+        ssl_verification_error = NextcloudSyncWorker.get_ssl_verification_error(e)
+        if ssl_verification_error is None:
             return False
-        max_retry_error = e.args[0]
-        ssl_error = max_retry_error.reason
-
-        # Get the ssl.SSLCertVerificationError
-        if not type(ssl_error.args) is tuple or len(ssl_error.args) != 1:
-            return False
-        if not isinstance(ssl_error.args[0], SSLCertVerificationError):
-            return False
-        ssl_verification_error = ssl_error.args[0]
 
         return (
             ssl_verification_error.verify_code
             in NextcloudSyncWorker.OPENSSL_SELF_SIGNED_CERTIFICATE_ERROR_CODES
         )
+
+    @staticmethod
+    def get_ssl_verification_error(
+        e: requests.exceptions.SSLError,
+    ) -> Optional[SSLCertVerificationError]:
+        # Get the requests SSLError
+        if not type(e.args) is tuple or len(e.args) != 1:
+            return None
+        max_retry_error = e.args[0]
+        ssl_error = max_retry_error.reason
+
+        # Get the ssl.SSLCertVerificationError
+        if not type(ssl_error.args) is tuple or len(ssl_error.args) != 1:
+            return None
+        if not isinstance(ssl_error.args[0], SSLCertVerificationError):
+            return None
+        return ssl_error.args[0]
