@@ -2,7 +2,7 @@ import itertools
 from typing import Dict, List, Optional
 
 from iotas.category import Category, CategorySpecialPurpose
-from iotas.database import Database, DbCursor, noaccents
+from iotas.database import Database, DbCursor
 from iotas.note import Note, DirtyFields
 
 
@@ -150,8 +150,8 @@ class NoteDatabase:
         """Fetch notes from the database by db ids.
 
         :param List[int] db_ids: Database id of the note
-        :return: The note, or None
-        :rtype: Optional[Note]
+        :return: The notes, in the same order as the provided ids
+        :rtype: List[Note]
         """
         if len(db_ids) == 0:
             return []
@@ -162,7 +162,8 @@ class NoteDatabase:
                 f"FROM note WHERE id IN ({','.join(['?']*len(db_ids))})"
             )
             result = sql.execute(request, db_ids)
-            return [NoteDatabase.db_row_to_note(row) for row in result]
+            as_dict = {row[8]: NoteDatabase.db_row_to_note(row) for row in result}
+            return [as_dict[note_id] for note_id in db_ids]
 
     def get_all_notes_remote_ids(self) -> List[int]:
         """Fetch remote ids for all remotely synced notes.
@@ -401,17 +402,18 @@ class NoteDatabase:
         :return: A list of ids for any matching notes
         :rtype: List[int]
         """
-        search = noaccents(term)
         with DbCursor(self.__db_base) as sql:
-            filters = ["%" + search + "%", "%" + search + "%"]
-            request = (
-                "SELECT rowid FROM note "
-                "WHERE (noaccents(title) LIKE ?"
-                "  OR noaccents(content) LIKE ?)"
-            )
+            filters = []
+            search = term.replace('"', '""')
+            search = search.replace("'", "''")
+            request = "SELECT i.rowid FROM note_fts_idx i "
+            if sort:
+                request += " INNER JOIN note n ON i.rowid = n.id "
+            request += " WHERE "
             if len(restrict_to) > 0:
-                request += f" AND id IN ({','.join(['?']*len(restrict_to))})"
+                request += f" id IN ({','.join(['?']*len(restrict_to))}) AND"
                 filters.extend(restrict_to)
+            request += f" note_fts_idx MATCH '\"{search}\"*' "
             if sort:
                 request += " ORDER BY last_modified DESC"
 
