@@ -178,5 +178,6 @@ class SelectionHeaderBar(Adw.Bin):
                 have_readonly = True
                 break
         if have_readonly:
-            show_error_dialog(self.get_root(), _("Unable to change category on read-only note"))
+            # Translators: Description, alert
+            show_error_dialog(self, _("Unable to change category on read-only note"))
         return have_readonly
