@@ -1,5 +1,5 @@
 from gettext import gettext as _
-from gi.repository import GObject, GLib, Gtk
+from gi.repository import Gio, GLib, GObject, Gtk
 
 import logging
 from threading import Lock, Thread
@@ -45,6 +45,7 @@ class SyncManager(GObject.Object):
         "secret-service-failure": (GObject.SignalFlags.RUN_FIRST, None, ()),
         "remote-category-sanitised": (GObject.SignalFlags.RUN_FIRST, None, (str, str)),
         "missing-password": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "notes-capability-missing": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
     STARTUP_SYNC_INIT_DURATION = 500
@@ -70,7 +71,8 @@ class SyncManager(GObject.Object):
         self.__sync_fail_counter = 0
 
         iotas.config_manager.settings.connect(
-            f"changed::{iotas.config_manager.SYNC_INTERVAL}", self.__set_sync_interval
+            f"changed::{iotas.config_manager.SYNC_INTERVAL}",
+            self.__on_sync_interval_setting_changed,
         )
         self.__set_sync_interval()
 
@@ -82,6 +84,7 @@ class SyncManager(GObject.Object):
             "authenticated", self, "authenticated", GObject.BindingFlags.SYNC_CREATE
         )
         self.__sync_worker.connect("missing-password", self.__on_missing_password)
+        self.__sync_worker.connect("notes-capability-missing", self.__on_missing_capability)
 
     def init_auth(self) -> None:
         """Initialise sync authentication details from the Secret Service."""
@@ -215,8 +218,9 @@ class SyncManager(GObject.Object):
 
         start_time = time.time()
         sync_result = self.__sync_worker.get_notes(force_refresh_all)
-        logging.debug("Server GET took {:.2f}s".format(time.time() - start_time))
-        if not sync_result.success:
+        if sync_result.success:
+            logging.debug("Server GET took {:.2f}s".format(time.time() - start_time))
+        else:
             logging.info("Refresh failed with status code %s", sync_result.status_code)
 
             # Ensure our API check gets run, just in case a major API version issue is causing
@@ -231,6 +235,7 @@ class SyncManager(GObject.Object):
             return
         self.__sync_fail_counter = 0
         if self.__offline:
+            logging.info("Recovered connectivity")
             self.offline = False
         notes = sync_result.data
         remote_ids_from_db = self.__db.get_all_notes_remote_ids()
@@ -479,11 +484,15 @@ class SyncManager(GObject.Object):
             self.sync_now()
 
     def __on_missing_password(self, _obj: GObject.Object) -> None:
-        logging.info(
-            "Sync is configured but password missing. Login to the server to reauthenticate."
-        )
+        if self.configured_but_no_password:
+            logging.info(
+                "Sync is configured but password missing. Login to the server to reauthenticate."
+            )
         self.emit("missing-password")
 
+    def __on_missing_capability(self, _obj: GObject.Object) -> None:
+        self.emit("notes-capability-missing")
+
     def __on_api_check_finished(self, _obj: GObject.Object) -> None:
         self.__api_check_complete = True
         thread = Thread(target=self.__sync_worker.fetch_settings)
@@ -504,6 +513,9 @@ class SyncManager(GObject.Object):
     def __on_secret_service_failure(self, _obj: GObject.Object) -> None:
         GLib.idle_add(self.emit, "secret-service-failure")
 
+    def __on_sync_interval_setting_changed(self, _settings: Gio.Settings, _key: str) -> None:
+        self.__set_sync_interval()
+
     def __api_check_and_settings_sync(self) -> None:
         # API check and settings sync are slung onto the back of the first request so that first
         # sync isn't as delayed. Justified by the fact that the outcome of the API check isn't
