@@ -77,50 +77,6 @@ def add_mouse_button_accel(
     widget._gesture_click = gesture
 
 
-def add_mouse_right_click_accel(
-    widget: Gtk.Widget,
-    function: Callable[[], None],
-    propagation: Gtk.PropagationPhase = Gtk.PropagationPhase.BUBBLE,
-) -> None:
-    """Add a mouse right click gesture.
-
-    :param Gtk.Widget widget: Widget to add on
-    :param Callable[[], None] function: Callback function
-    :param Gtk.PropagationPhase propagation: Propagation phase
-    """
-
-    def callback(gesture: Gtk.GestureClick, _n_press: int, _x: float, _y: float) -> None:
-        if gesture.get_current_button() == 3:  # Right click
-            function()
-
-    add_mouse_button_accel(widget, callback, propagation)
-
-
-def add_touch_longpress_accel(
-    widget: Gtk.Widget,
-    function: Callable[[], None],
-    propagation: Gtk.PropagationPhase = Gtk.PropagationPhase.BUBBLE,
-) -> None:
-    """Add a long press touch gesture.
-
-    :param Gtk.Widget widget: Widget to add on
-    :param Callable[[], None] function: Callback function
-    :param Gtk.PropagationPhase propagation: Propagation phase
-    """
-    gesture = Gtk.GestureLongPress.new()
-    gesture.set_propagation_phase(propagation)
-    gesture.set_touch_only(True)
-
-    def longpress(_gesture: Gtk.GestureLongPress, _x: float, _y: float) -> None:
-        function()
-
-    gesture.connect("pressed", longpress)
-
-    widget.add_controller(gesture)
-    # Keep the gesture in scope
-    widget._gesture_longpress = gesture
-
-
 def idle_add_wait(function, *args, **kwargs) -> Any:
     """Execute function in the GLib main loop and wait.
 
@@ -144,18 +100,18 @@ def idle_add_wait(function, *args, **kwargs) -> Any:
     return results.pop()
 
 
-def show_error_dialog(parent: Gtk.Window, message: str) -> None:
+def show_error_dialog(parent: Gtk.Widget, message: str) -> None:
     """Show a modal error dialog.
 
-    :param Gtk.Window parent: Parent window
+    :param Gtk.Widget parent: Parent widget
     :param str message: Message text
     """
     # Translators: Title
-    dialog = Adw.MessageDialog.new(parent, _("Error"), message)
+    dialog = Adw.AlertDialog.new(_("Error"), message)
     # Translators: Button
     dialog.add_response("close", _("Close"))
     dialog.set_close_response("close")
-    dialog.show()
+    dialog.present(parent)
 
 
 def check_for_search_starting(
