@@ -10,7 +10,7 @@ from iotas.index import Index
 from iotas.note import Note
 from iotas.note_database import NoteDatabase
 from iotas.note_manager import NoteManager
-from iotas.nextcloud_login_window import NextcloudLoginWindow, LoginWindowMode
+from iotas.nextcloud_login_dialog import NextcloudLoginDialog, LoginDialogMode
 from iotas.sync_manager import SyncManager
 
 
@@ -72,14 +72,13 @@ class Window(Adw.ApplicationWindow):
         )
 
         self.connect("close-request", self.__on_close_request)
-
-        # Part of workaround for https://gitlab.gnome.org/GNOME/gtk/-/issues/6405
-        # TODO remove once resolved
-        self.connect("notify::is-active", self.__on_is_active_changed)
+        self.connect("notify::visible-dialog", self.__update_for_visible_dialogs)
 
         (width, height) = iotas.config_manager.get_window_size()
         self.set_default_size(width, height)
-        self._index.update_layout_for_initial_width(width)
+        self._index.update_layout_for_initial_size(width, height)
+        self.connect("notify::default-height", self.__on_height_changed)
+
         self._index.active = True
         self._editor.active = False
 
@@ -115,6 +114,12 @@ class Window(Adw.ApplicationWindow):
         action_group.add_action(action)
         app.set_accels_for_action("win.fullscreen", ["F11"])
 
+        action = Gio.SimpleAction.new("close")
+        action.connect("activate", self.__on_close_window)
+        action_group.add_action(action)
+        app.set_accels_for_action("win.close", ["<Control>w"])
+        self.__close_window_action = action
+
         action = Gio.SimpleAction.new("refresh")
         action.connect("activate", self.__on_refresh)
         action_group.add_action(action)
@@ -200,17 +205,16 @@ class Window(Adw.ApplicationWindow):
         else:
             self.fullscreen()
 
+    def __on_close_window(
+        self,
+        _obj: GObject.Object,
+        _value: GObject.ParamSpec,
+    ) -> None:
+        self.__cleanup_and_close()
+
     def __on_close_request(self, _obj: GObject.Object) -> None:
         """On window destroyed."""
-        self.__sync_manager.close()
-        if not self.is_fullscreen():
-            iotas.config_manager.set_window_size(self.get_width(), self.get_height())
-        iotas.config_manager.set_first_start(False)
-        self.close()
-
-    def __on_is_active_changed(self, window: Gtk.Window, _param: GObject.ParamSpec) -> None:
-        if window.is_active() and self._editor.active:
-            self._editor.focus_textview_if_editing()
+        self.__cleanup_and_close()
 
     def __on_secret_service_failure(self, _obj: GObject.Object) -> None:
         self.__secret_service_failed = True
@@ -278,6 +282,9 @@ class Window(Adw.ApplicationWindow):
         elif self.__open_note_after_init is not None:
             self.__open_note_by_id(self.__open_note_after_init, True)
 
+    def __on_height_changed(self, _obj: Gtk.Window, _value: GObject.ParamSpec) -> None:
+        self._index.update_search_pagesize_for_height(self.get_height())
+
     def __open_note(self, note: Note, immediate: bool) -> None:
         if immediate:
             self._navigation.set_animate_transitions(False)
@@ -300,11 +307,23 @@ class Window(Adw.ApplicationWindow):
 
     def __show_login_window(self) -> None:
         if self.__secret_service_failed:
-            mode = LoginWindowMode.SECRET_SERVICE_FAILURE
+            mode = LoginDialogMode.SECRET_SERVICE_FAILURE
         elif self.__sync_manager.configured_but_no_password:
-            mode = LoginWindowMode.REAUTHENTICATE
+            mode = LoginDialogMode.REAUTHENTICATE
         else:
-            mode = LoginWindowMode.NORMAL
-        window = NextcloudLoginWindow(self.__sync_manager, mode)
-        window.set_transient_for(self)
-        window.present()
+            mode = LoginDialogMode.NORMAL
+        window = NextcloudLoginDialog(self.__sync_manager, mode)
+        window.present(self)
+
+    def __update_for_visible_dialogs(self, window, _value: GObject.ParamSpec) -> None:
+        visible = window.get_visible_dialog() is not None
+        self._index.update_for_dialog_visibility(visible)
+        self._editor.update_for_dialog_visibility(visible)
+        self.__close_window_action.set_enabled(not visible)
+
+    def __cleanup_and_close(self) -> None:
+        self.__sync_manager.close()
+        if not self.is_fullscreen():
+            iotas.config_manager.set_window_size(self.get_width(), self.get_height())
+        iotas.config_manager.set_first_start(False)
+        self.close()
